#!/bin/sh
#
# $Id: bin/ipv4Manager.sh 1.1.2.5 2007/08/08 08:31:53MDT Brian Szmyd (bszmyd) dev  $
##

## Path to application home
applicationHome="/home/embedded/library"
## Source include file.
. ${applicationHome}/AppManager/bin/include.sh
. ${applicationHome}/AppManager/bin/serviceManager.sh

eth2interfaceConfigurationFile="/etc/network/interfaceEth2"
STATIC_ADDRESS_INFO=""
EXTERNAL_INTERFACE="eth2"

## Error Codes:
EXIT_STATUS_WRONG_FORMAT=5


#########################################
# getStaticAddressInfo
#
# Inputs: None
# Output: Populates the variable STATIC_ADDRESS_INFO with the static IP,
# netmask and gateway found in the file /etc/network/interfaceEth2. It has the
# following format "192.168.1.1?255.255.255.0?192.168.1.4"
#
#########################################
getStaticAddressInfo()
{
   STATIC_ADDRESS_INFO=`cat $eth2interfaceConfigurationFile | awk ' 
   BEGIN { startProcessing=0; staticIP=""; netmask=""; gateway=""; }
   {
      inet=$3
      static=$4

      if( $3 == "inet" && $4 == "static" ) 
      {
         startProcessing=1
         next
      }

      if( startProcessing == 1 ) 
      {
         staticIP=$2
         startProcessing++;
         next
      }
      if( startProcessing == 2 ) 
      {
         netmask=$2
         startProcessing++;
         next
      }
      if( startProcessing == 3 ) 
      {
         gateway=$2
         startProcessing++;
      }
   } END { printf("%s?%s?%s",staticIP, netmask, gateway) }'`
   COMMAND_EXIT_STATUS=$?
   if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
      LogTrace "Get Static Address Information: $STATIC_ADDRESS_INFO"
      if [ "$STATIC_ADDRESS_INFO" != "??" ]; then
         echo "$STATIC_ADDRESS_INFO"
      fi
   else
      LogError "Failed to get Static Address Information: $STATIC_ADDRESS_INFO"
   fi
}

#########################################
# deleteStaticAddress
# Deletes the static IP information from the "/etc/network/interfaceEth2"
# file. This includes the address, netmask and gateway.
#
# Inputs: flag to restart the network interface
#########################################
deleteStaticAddress()
{
   LogTrace "Delete Static Address"
   local tmpFile="/tmp/staticIPfile.$$"
   local restart="$1"
   local address=`grep 'address.*\..*' /etc/network/interfaceEth2 | awk '{print $2}'`

   if [ "X${address}" != "X" ]; then
      address=`ifconfig eth2 | grep " addr:${address}"`
   fi

   if [ $? -ne 0 ]; then
      address="";
   fi

   if [ -f "$tmpFile" ]; then
      RunCommand "rm $tmpFile"
   fi
   RunCommand "touch $tmpFile"
   cat $eth2interfaceConfigurationFile | awk ' 
   { 
      inet=$3
      static=$4

      if( $3 == "inet" && $4 == "static" ) 
      {
         startProcessing=1
         next
      }

      if( !(startProcessing > 0 && startProcessing < 4) ) 
      {
         printf("%s\n", $0);
         next
      }
      else if( startProcessing == 1 ) 
      {
         startProcessing++;
         next
      }
      else if( startProcessing == 2 ) 
      {
         startProcessing++;
         next
      }
      else if( startProcessing == 3 ) 
      {
         startProcessing++;
      }
   } ' >> $tmpFile
   COMMAND_EXIT_STATUS=$?
   if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
      RunCommand "mv $tmpFile $eth2interfaceConfigurationFile";
      ## if [ "X${address}" != "X" ]; then
         ## RunCommand "ip addr del $address dev eth2"
      ## fi
      if [ "X$restart" != "X" ]; then
         restartNetwork
      fi
   fi
}

#########################################
# configureStaticAddress
# Configure Static IP. This deletes the existing entry in eth2 configuration
# file and then adds the new entry
#
# Inputs: New IP, gateway.
#########################################
configureStaticAddress()
{
   local addressNetmaskGateway="$1"
   local address=""
   local netmask=""
   local gateway=""
   
   if [ "$addressNetmaskGateway" = "delete" ]; then
      enableDHCP "on"
   else
      address=`echo $addressNetmaskGateway | awk -F? '{print $1}'`
      netmask=`echo $addressNetmaskGateway | awk -F? '{print $2}'`
      gateway=`echo $addressNetmaskGateway | awk -F? '{print $3}'`

      LogTrace "Configure Static Address - address: $address, netmask: $netmask, gateway: $gateway"
      if [ "X$address" = "X" -o "X$netmask" = "X" -o "X$gateway" = "X" ]; then
         LogError "Invalid format"
         COMMAND_EXIT_STATUS=$EXIT_STATUS_WRONG_FORMAT
      fi

      if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
         deleteStaticAddress
         if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
            enableDHCP "off"
            echo "iface eth2 inet static" >> $eth2interfaceConfigurationFile
            echo "     address $address" >> $eth2interfaceConfigurationFile
            echo "     netmask $netmask" >> $eth2interfaceConfigurationFile
            echo "     gateway $gateway" >> $eth2interfaceConfigurationFile
            restartNetwork
            checkForAddress
         fi
      fi
   fi
}

#########################################
# enableDHCP
# Enable/Disable DHCP.
#
# Inputs: "on" or "off"
#########################################
enableDHCP()
{
   local option="$1"
   LogTrace "Enable DHCP: $option"
   if [ "$option" = "on" ]; then
      deleteStaticAddress 
      removeDHCP
      echo "iface eth2 inet dhcp" >> $eth2interfaceConfigurationFile
      echo "      hostname \`hostname\`" >> $eth2interfaceConfigurationFile
      restartNetwork
   elif [ "$option" = "off" ]; then
      stopNetwork
      removeDHCP
      startNetwork
   else
      Usage
   fi
   if [ "$option" = "on" ]; then
      checkForAddress
   fi
}

#########################################
# removeDHCP
# Removes DHCP entry from interface file.
#
#########################################
removeDHCP()
{
   LogTrace "Remove DHCP entry"
   local tmpFile="/tmp/staticIPfile.$$"

   if [ -f "$tmpFile" ]; then
      RunCommand "rm $tmpFile"
   fi
   RunCommand "touch $tmpFile"
   cat $eth2interfaceConfigurationFile | awk ' 
   { 
      inet=$3
      dhcp=$4

      if( $3 == "inet" && $4 == "dhcp" ) 
      {
         startProcessing=1
         next
      }

      if( !(startProcessing > 0 && startProcessing < 2) ) 
      {
         printf("%s\n", $0);
         next
      }
      else if( startProcessing == 1 ) 
      {
         startProcessing++;
         next
      }
   } ' >> $tmpFile
   COMMAND_EXIT_STATUS=$?
   if [ $COMMAND_EXIT_STATUS -eq 0 ]; then
      RunCommand "mv $tmpFile $eth2interfaceConfigurationFile";
   fi
}

#########################################
# changeLibraryHostname
# Change the library's host name. Updates the /etc/hosts and /etc/hostname files.
#
# Inputs: New hostname
#
#########################################
changeLibraryHostname()
{
   local newHostname="$1"
   local currentHostname=`hostname`

   RunCommand "sed -i \"s/$currentHostname/$newHostname/\" /etc/hosts /etc/hostname"
   RunCommand "hostname $newHostname"
}

#########################################
# getLibraryHostname
# Get the library hostname
#
#########################################
getLibraryHostname()
{
   local hostname=`hostname`
   COMMAND_EXIT_STATUS=$?
   echo $hostname
}

#########################################
# isDHCPenabled
#
# Return "??" if DHCP is disabled otherwize "IP?MASH?GATEWAY"
#########################################
isDHCPenabled()
{
   local address=""
   local netmask=""
   local gateway=""
   grep 'iface eth2 inet dhcp' /etc/network/interfaceEth2 >/dev/null 2>&1
   if [ $? -eq 0 ]; then
      address=`ifconfig eth2 | grep 'inet addr:' | awk '{print $2}' | awk -F: '{print $2}'`
      netmask=`ifconfig eth2 | grep 'inet addr:' | awk '{print $4}' | awk -F: '{print $2}'`
      gateway=`route | grep default | awk '{print $2}'`
   fi
   echo "${address}?${netmask}?${gateway}"
}

#########################################
# checkForAddress
#
# Verify interface got an address
#########################################
checkForAddress()
{
   local timeout=0
   while [ 1 ]; do
      ifconfig eth2 | grep 'inet addr:' >/dev/null 2>&1
      if [ $? -eq 0 ]; then
         COMMAND_EXIT_STATUS=0
         break
      else
         sleep 1
         timeout=`expr $timeout + 1`
         if [ $timeout -gt 120 ]; then
            LogTrace "Failed to get address for interface eth2"
            COMMAND_EXIT_STATUS=32
            break
         fi
      fi
   done
}


#########################################
# Display usage message
#
# Output: Display usage message and exit the script.
#########################################
Usage()
{
   echo "Usage: $0 < Action > [ Options ]   

   Actions:

     -c             : configure IP address, requires a -s or -d option

     -g \"option\"    : get information, where \"option\" is one of the following

        static - get static address, netmask and gateway info
        host   - get the library hostname
        dns    - get the DNS servers
        ntp    - get the NTP servers

     -h             : show this message

     -n \"hostname\"  : change library hostname, requires a hosname argument
      
     -r             : remove static address

     -D \"servers\"   : set the DNS addresses with the following format

        <server 1>[ <server 2> ...]
                    
     -t \"servers\"   : set the NTP time servers with the following format

        <server 1>[ <server 2> ...]

                      disable with \"disable\"

   Options:

     -d \"state\"     : dhcp, states are \"on\", \"off\" or  \"status\"

     -s             : configure Static address (used with -c), where static
                      address, netmask and gateway are provide in the following format

        192.168.1.12?255.255.255.0?192.168.1.1

        notice that the \"?\" delimites the IP from the netmask and gateway.

        To delete this address -s \"delete\"

   Examples:

      configure static IP 192.168.10.4 with netmask 255.255.248.0 and gateway 192.168.1.1
         $0 -c -s \"192.168.10.4?255.255.248.0?192.168.1.1\"

      enable dhcp
         $0 -c -d \"on\"

      get static address info
         $0 -g static

      set ntp servers
         $0 -t "192.168.30.1 172.233.0.1"
   ";
   exit 1
}

#################################################################################
####                               M A I N                                   ####
#################################################################################

operation=""
getAddress=""
removeStaticAddress=""
configureAddress=""
configureDHCP=""
changeHostname=""
newHostname=""


if [ $# -eq 0 ]; then
   Usage
else
   while getopts "cd:g:hn:rs:D:t:" opt
   do
      case "$opt" in
         c) configureAddress="true"
         ;;
         d) configureDHCP="${OPTARG}"
         ;;
         h) Usage
         ;;
         g) operation="${OPTARG}"
         getAddress="true"
         ;;
         n) newHostname="${OPTARG}"
         changeHostname="true"
         ;;
         r) removeStaticAddress="true"
         ;;
         s) staticAddress="${OPTARG}"
         ;;
         t) ntpServers="${OPTARG}"
         ;;
         D) domainServers="${OPTARG}"
         ;;
         *) Usage
         ;;
      esac
   done
fi

if [ "$configureAddress" = "true" ]; then
   if [ "$configureDHCP" = "on" -o "$configureDHCP" = "off" ]; then
      enableDHCP "$configureDHCP"
   elif [ "X$staticAddress" != "X" ]; then
      configureStaticAddress "$staticAddress"
   else
      Usage
   fi
elif [ "$getAddress" = "true" ]; then
   if [ "$operation" = "static" ]; then
      getStaticAddressInfo
   elif [ "$operation" = "host" ]; then
      getLibraryHostname
   elif [ "$operation" = "dns" ] ; then
      getDomainServers
   elif [ "$operation" = "ntp" ] ; then
      getNTPServers
   else
      Usage
   fi
elif [ "$removeStaticAddress" = "true" ]; then
   deleteStaticAddress "restart"
elif [ "$changeHostname" = "true" ]; then
   if [ "X$newHostname" = "X" ]; then
      Usage
   fi
   changeLibraryHostname $newHostname
elif [ "$configureDHCP" = "status" ]; then
   isDHCPenabled
elif [ -n "${domainServers}" ]; then
    configureDNS "${domainServers}"
elif [ -n "${ntpServers}" ]; then
    configureNTP "${ntpServers}"
else
   Usage
fi

LogTrace "Exit status: $COMMAND_EXIT_STATUS"
exit $COMMAND_EXIT_STATUS
